      program reference_elev
c
c  Modify vacant allotment and Canadian entries in the main engineering
c  data file to make the HAAT and RCAMSL values consistent.  The
c  elevation AMSL from 2 to 10 miles along uniformly spaced radials is
c  calculated as a reference elevation, and the station RCAMSL is then
c  set equal to this reference plus the station HAAT.  If, as expected,
c  the HAAT of a vacant allotment is 0.0, the RCAMSL will be set equal
c  to the reference elevation.
c
c  Set azimuth increment as appropriate.  It must be 45 degrees or
c  smaller and must divide 360 degrees exactly.
c
      character*48 eng_db /'eng_db.unf'/
      logical issue_warning /.false./
      parameter(
     &   AZ_INC = 45.0,
     &   NUMRAD = 360.0/AZ_INC,
     &   MINRAD = 0.75*NUMRAD)
c
      include 'global.inc'
      character svc_cls*2, cty*1, state*2, city*20, call*8
      character pol*1, da*1, beamt*1, zone*1, antmnf*3
      character antmod*14, off*1, arn*8
      common /mainrec/
     &   latsec, lonsec, erp, haat, rcamsl, ref_az, itype, istat,
     &   svc_cls, off, pol, da, beamt, zone, antmnf, antmod,
     &   ichan, cty, city, state, call, arn
c
      double precision dlat, dlon
      data compute_haat /.true./
c
c  Load terrain data index and initialize array pointers
c
      call get_ter_dirs
      call load_ter_index
      call init_ter_mem
      next_pt = 1
c
c  Open engineering data base
c
      call getlun(imain)
      open(unit=imain, file=eng_db, status='old', form='unformatted',
     &   access='direct', recl=99, iostat=ierr)
      if (ierr .ne. 0) then
         call perror(eng_db)
         call exit(ierr)
      end if
c
c  Loop thru records selecting those that should be modified
c
      num_canadian = 0
      num_vacant_usa = 0
      num_others = 0
      num_failures = 0
      irec = 0
   20 continue
      irec = irec + 1
      if (mod(irec, 100) .eq. 0) write(0, '(a,i4)') 'At record ', irec
      call flush(0)
      call rd_main(imain, irec)
      if (arn(1:7) .eq. 'atv_rec') then
         write(0, '(2a)') 'ATV record found in ',
     &      eng_db(1:lnblnk(eng_db))
         write(0, '(a)')
     &     'Execute add_reference_elev before adding ATV assignments'
         call exit(257)
      end if
      if (city(1:11) .eq. 'end of file') goto 200
      if (rcamsl .ne. 0.0) then
         if (haat .eq. 0.0 .and. issue_warning) write(0, 1000)
     &      eng_db(1:lnblnk(eng_db)), cty, state, city(1:lnblnk(city)),
     &      ichan, haat, rcamsl, eng_db(1:lnblnk(eng_db))
         goto 20
      end if
c
c  Skip Puerto Rico, Virgin Islands, longitudes outside a certain range,
c  and Mexican stations.
c
      if (state .eq. 'PR' .or. state .eq. 'VI' .or. state .eq. 'AK'
     &   .or. cty .eq. 'M') goto 20
      call sec_to_dpdeg(lonsec, dlon)
      if (dlon .gt. 126.0 .or. dlon .lt. 52.0) goto 20
      call sec_to_dpdeg(latsec, dlat)
c
c  Begin loop over uniformly spaced radials
c
      az = -AZ_INC
      sum = 0.0
      icount = 0
      do j=1, NUMRAD
         az = az + AZ_INC
c
c  Get average elevation of radial.  If too many errors occured getting
c  the terrain data, rad_haat is set to -10000.0.  Rad_haat is also set
c  to -10000.0 if routine get_rad_haat finds logical switch compute_haat
c  is false, so this switch should normally be set true in the present
c  program.
c
         call get_rad_haat(az, rcamsl, dlat, dlon, elev, rad_haat)
         if (rad_haat .ne. -10000.0) then
            sum = sum + elev
            icount = icount + 1
         end if
      end do
c
c  Calculate reference elevation
c
      if (icount .ge. MINRAD) then
         ref = sum / icount
         revised = ref + haat
c
c  Write modified data base record
c
         write(imain, rec=irec, err=100)
     &      ichan, cty, city, state, call, latsec, lonsec,
     &      erp, haat, revised, ref_az, itype, istat, svc_cls,
     &      off, pol, da, beamt, zone, antmnf, antmod, arn
c
         if (cty .eq. 'C') then
            num_canadian = num_canadian + 1
         else if (cty .eq. 'A' .and. istat .eq. 4) then
            num_vacant_usa = num_vacant_usa + 1
         else
            num_others = num_others + 1
         end if
         if (istat .lt. 4) write(0, '(a,i3,3(1x,a),f7.1)') 'RCAMSL of',
     &      ichan, state, city(1:lnblnk(city)), 'changed from 0.0 to',
     &      revised
      else
         write(0, '(a,i3,2(1x,a))')
     &      'Insuffient data to revise RCAMSL of', ichan, state,
     &      city(1:lnblnk(city))
         num_failures = num_failures + 1
      end if
      goto 20
c
  100 continue
      write(0, '(2a)') 'Error writing to file ',
     &   eng_db(1:lnblnk(eng_db))
  200 continue
      close(imain)
      write(0, 1010) num_canadian, num_vacant_usa, num_others,
     &   num_failures
c
 1000 format(
     &   'Incompatible HAAT and RCAMSL found in ', a, ' for'/
     &   a1, 1x, a2, 1x, a, ' channel', i3/
     &   'HAAT =', f7.1, ', RCAMSL =', f7.1/ 'File ', a,
     &   ' may already have been run through reference_elev')
 1010 format(
     &   'The RCAMSL has been revised for the following:'/
     &   i5, ' Canadian records'/
     &   i5, ' U.S. vacant allotments'/
     &   i5, ' other records'/
     &   'Terrain data was lacking for the modification of', i5,
     &   ' records')
      end
