c***********************************************************************
c
c                         create_haat_file
c
c  This program creates a file of the HAATs at regularly spaced azimuths
c  for each U.S. station in the TV Engineering data base, and optionally
c  for Canadian stations too.  (The usual purpose of these data is the
c  determination of grade B contours for replication, so only U.S.
c  stations are of interest.) The HAAT records are created in the same
c  order as the records in the engineering data base.  An index file is
c  also created to make it possible to retrieve HAAT records by station
c  name.
c
c  No HAAT records are created for the paired ATV stations.  It is assumed
c  they would be the same as for the NTSC station.
c
c***********************************************************************
c
c  Set filenames and azimuth increment as appropriate.  Azimuth increment
c  must be 45 degrees or smaller and must divide 360 degrees exactly.
c
      parameter(
     &   AZ_INC = 5.0,
     &   NUMRAD = 360.0/AZ_INC)
      character*48
     &   haat_file /'haat_db.dbs'/,
     &   haat_index_file /'haat_db.idx'/,
     &   eng_db /'tv_main.dbs'/
c
      include 'global.inc'
      character svc_cls*2, cty*1, state*2, city*20, call*8
      character pol*1, da*1, beamt*1, zone*1, antmnf*3
      character antmod*14, off*1, arn*8
      common /mainrec/
     &   latsec, lonsec, erp, haat, rcamsl, ref_az, itype, istat,
     &   svc_cls, off, pol, da, beamt, zone, antmnf, antmod,
     &   ichan, cty, city, state, call, arn
      integer lat(3), long(3)
c
      double precision dlat, dlon
      logical atv_rec
      logical*1 handle /.false./
      logical canada_too /.false./
      data compute_haat /.true./
c
      external handler
c
c  Equate routine_name for error reporting
c
      routine_name = 'create_haat_file'
c
c  Set IEEE floating point exception handlers
c
      if (handle) then
         ieeer1 = ieee_handler('set', 'division', handler)
         ieeer2 = ieee_handler('set', 'overflow', handler)
         ieeer3 = ieee_handler('set', 'invalid', handler)
      end if
c
c  Load terrain data index and initialize array pointers
c
      call get_ter_dirs
      call load_ter_index
      call init_ter_mem
      next_pt = 1
c
c  Open engineering data base
c
      call getlun(imain)
      open(unit=imain, file=eng_db, status='old', form='unformatted',
     &   access='direct', recl=99, iostat=ierr)
      if (ierr .ne. 0) then
         call perror(eng_db)
         call exit(ierr)
      end if
c
c  Open output files
c
      n_size = (NUMRAD * 4) + 4
      call getlun(ihaat_file)
      open(unit=ihaat_file, file=haat_file, status='new',
     &   form='unformatted', access='direct', recl=n_size, iostat=ierr)
      if (ierr .ne. 0) then
         call perror(haat_file)
         call exit(ierr)
      end if
c
      call getlun(index_file)
      open(unit=index_file, file=haat_index_file, status='new',
     &   form='formatted', access='sequential', iostat=ierr)
      if (ierr .ne. 0) then
         call perror(haat_index_file)
         call exit(ierr)
      end if
c
c  Store increment as first record in HAAT file.  This will be used to
c  determine how many HAATs to read for each record.
c
      write(ihaat_file, rec=1) AZ_INC
c
c  Loop thru records and get HAAT at specified increments
c
      irec = 0
   20 continue
      irec = irec + 1
      call rd_main(imain, irec)
      if (cty .ne. 'A' .and. .not. canada_too) goto 100
      atv_rec = call(1:1) .eq. 'D' .or. index(arn, 'atv_rec') .gt. 0
      if (atv_rec) goto 100
      if (city(1:11) .eq. 'end of file') goto 100
c
c  Skip Puerto Rico, Virgin Islands and longitudes outside a certain
c  range.  Skipped records will have the single value of HAAT as read
c  from the data base stored for each azimuth.
c
      call sec_to_dpdeg(latsec, dlat)
      call sec_to_dpdeg(lonsec, dlon)
c
      if (state .eq. 'PR' .or. state .eq. 'VI' .or. state .eq. 'AK'
     &   .or. dlon .gt. 140. .or. dlon .lt. 52) then
         do l=1, NUMRAD
            radial_haat(l) = haat
            end do
         goto 90
      end if
c
      az = -AZ_INC
      do j=1, NUMRAD
         az = az + AZ_INC
c
c  Get HAAT for radial.  If too many errors occured getting the terrain
c  data, rad_haat is set to -10000.0.  See routine get_rad_haat for
c  more info.  Also rad_haat is set to -10000.0 if routine get_rad_haat
c  finds logical switch compute_haat is false.  This switch should
c  normally be set true in the present program.
c
         call get_rad_haat(az, rcamsl, dlat, dlon, avg_elev, rad_haat)
         if (rad_haat .eq. -10000.0) rad_haat = haat
         radial_haat(j) = rad_haat
      end do
c
c  Write data base record and companion line of index file
c
   90 continue
      write(ihaat_file, rec=irec+1) rcamsl,
     &   (radial_haat(k), k=1, NUMRAD)
      call hms(latsec, lat)
      call hms(lonsec, long)
      write(index_file, 1000) cty, state, city, lat, long, ichan
      if (mod(irec, 10) .eq. 0) then
         write(0, '(a,i4)') 'At record ', irec
         call flush(0)
      end if
      goto 20
c
  100 continue
      close(imain)
      close(ihaat_file)
      close(index_file)
c
c  Format of the HAAT index file
c
c1234567890123456789012345678901234567890123
cA CT BRIDGEPORT           411048 0731120 43
 1000 format(a1, 1x, a2, 1x, a20, t27, 3i2.2, 1x, i3.3, 2i2.2, i3)
      end

      subroutine hms(total_seconds, components)
      integer components(3), total_seconds
      components(1) = total_seconds/3600
      components(2) = mod(total_seconds, 3600)/60
      components(3) = mod(total_seconds, 60)
      return
      end
