c
c***********************************************************************
c
c                            deg_to_sec
c
c  Converts geographic coordinate point in decimal degrees to seconds
c
c***********************************************************************
c
      subroutine deg_to_sec(deg, isec)
      x = abs(deg)
      isec = int ((x * 3600.0) + 0.5)
      if (deg .lt. 0) isec = -isec
      return
      end
c  %W% %G%
c
c***********************************************************************
c
c                           get_3sec_corners
c
c  This routine determines the coordinates of the four corners of the 3
c  arc-second box containing the input point coordinates.
c
c***********************************************************************
c
      subroutine get_3sec_corners(pt_lat, pt_lon,
     &    pt_corner_lat, pt_corner_lon )
      double precision pt_lat, pt_lon
      double precision pt_corner_lat(4), pt_corner_lon(4)
c
      include 'global.inc'
c
      double precision pts_per_deg_lat, pts_per_deg_lon
      character saved_routine_name*32
c
c  Equate routine_name for error reporting
c
      saved_routine_name = routine_name
      routine_name = 'free_space'
c
c  Set the points per degree to 3 seconds.  Points have been added
c  above 50 degrees latitude where the original terrain data provided
c  points every 6 seconds.
c
      pts_per_deg_lat = 1200.0
      pts_per_deg_lon = 1200.0
c
c  Find corners in degrees
c
      pt_corner_lat(1) =
     &   int(pt_lat * pts_per_deg_lat ) / pts_per_deg_lat
      pt_corner_lat(2) = pt_corner_lat(1) + 1.0 / pts_per_deg_lat
      pt_corner_lat(3) = pt_corner_lat(2)
      pt_corner_lat(4) = pt_corner_lat(1)
c
      pt_corner_lon(1) =
     &   int(pt_lon * pts_per_deg_lon ) / pts_per_deg_lon
      pt_corner_lon(3) = pt_corner_lon(1) + 1.0 / pts_per_deg_lon
      pt_corner_lon(2) = pt_corner_lon(1)
      pt_corner_lon(4) = pt_corner_lon(3)
c
      routine_name = saved_routine_name
      return
      end
c  %W% %G%
c
c***********************************************************************
c
c                           get_3sec_elevation
c
c  Given a point in NAD83 coordinates, find the elevation at that point
c  using 3 arc sec terrain data.
c
c***********************************************************************
c
      subroutine get_3sec_elevation(d_pt_lat, d_pt_lon, elev)
      double precision d_pt_lat, d_pt_lon
      include 'global.inc'
c
      double precision pd
c
      pt_lat = d_pt_lat
      pt_lon = d_pt_lon
c
c  Find the index of the 7.5' block containing the given point
c
      call get_terrain_index(pt_lat, pt_lon, num_f, index)
c
c  Determine if this 7.5' block is in memory
c
      if (terrain_mem(index) .ne. 0) then
         n_loc = terrain_mem(index)
         call get_se_corner(pt_lat, pt_lon, se_lat, se_lon)
c
c  Terrain block not now in memory so it must be loaded
c
      else
         map_loads = map_loads + 1
         call get_se_corner(pt_lat, pt_lon, se_lat, se_lon)
         call load_75_terrain(se_lat, se_lon)
         n_loc = next_pt
      end if
c
c  Increment time stamp
c
      next_time = next_time + 1
      i_time(n_loc) = next_time
c
c  Check for no terrain data available and set elevation to -10000
c  this will be used by other routines to flag points that do not
c  have elevation data availble
c
      if (i_elev(n_loc, 1) .eq. -10000.0) then
         elev = -10000.0
         goto 500
      end if
c
c  Get elevation at point
c
      pd = 1200.0
c
      iy = 1 + int((d_pt_lat - se_lat) * pd)
      ix = int((d_pt_lon - se_lon) * pd)
c
      ip = iy + (ix * 150)
      elev = float(i_elev(n_loc, ip))
c
  500 return
      end
c  %W% %G%
c
c***********************************************************************
c
c                         get_nad_83_coord
c
c  This routine determines the NAD83 coordinates of a point given the
c  NAD27 coordinates.
c
c***********************************************************************
c
      subroutine get_nad_83_coord(pt_lat_27, pt_lon_27,
     &   pt_lat_83, pt_lon_83, n_error_code )
      integer*2 ns_bias, ew_bias, num_bits, elevation_bias
c
c  Initialize error code to 0
c
      n_error_code = 0
c
c  Initialize NAD83 coords to impossible values to flag lack of data
c
      pt_lat_83 = -100.0
      pt_lon_83 = -200.0
c
c  Estimate NAD83 coords
c
      est_lat = pt_lat_27
      est_lon = pt_lon_27
c
c  Initialize previous index
c
      idx_p = -1
      iterations = 0
c
c  Get southeast corner of 7.5 minute block containing the point
c
      do while (iterations .le. 3)
         call get_se_corner(est_lat, est_lon, se_lat, se_lon)
c
c  Get index
c
         call get_terrain_index(se_lat, se_lon, num_f, idx)
c
c  The correct block has been found if the index has not changed
c
         if (idx .eq. idx_p) goto 500
c
c  Get parameters for this 7.5 minute block.  Error code 3 means no data.
c
         call get_75_parms(idx, ns_bias, ew_bias, elevation_bias,
     &      num_bits)
         if (num_bits .eq. 0) then
            n_error_code = 3
            goto 510
         end if
c
c  Add bias to form new estimate
c
         est_lat = pt_lat_27 + (float(ns_bias) / 1.0e6 )
         est_lon = pt_lon_27 + (float(ew_bias) / 1.0e6 )
         iterations = iterations + 1
         idx_p = idx
      end do
c
c  Too many iterations
c
      if (warn) write(0, 1000) pt_lat_27, pt_lon_27
c
c  Normal exit
c
  500 continue
      pt_lat_83 = est_lat
      pt_lon_83 = est_lon
c
c  Exit when no data was found for one of the trial blocks
c
  510 continue
      return
c
 1000 format('Trouble converting', f8.4, 'N,', f9.4,
     &   'W to NAD83 coordinates')
      end
c  %W% %G%
c
c***********************************************************************
c
c                          get_se_corner
c
c  This routine determines the coordinates of the southeast corner of
c  a 7.5 minute block for a given point (pt_lat, pt_lon).  Coordinates
c  are in decimal degrees.
c
c***********************************************************************
c
      subroutine get_se_corner(pt_lat, pt_lon, se_lat, se_lon)
c
c  Determine the southeast corner coordinates of the 7.5 minute block
c  containing the given point
c
      se_lat = int(8.0 * pt_lat) / 8.0
      se_lon = int(8.0 * pt_lon) / 8.0
c
      return
      end
c  %W% %G%
c
c***********************************************************************
c
c                      get_ter_dirs
c
c  This routine reads the file terrain_directory.dbs to determine where
c  terrain data is stored.  The file contains a list of the directories
c  and the longitude of the western-most block stored in the directory.
c  The directory in which files are stored is determined by the
c  longitude of the terrain block.
c
c***********************************************************************
c
      subroutine get_ter_dirs
      include 'global.inc'
      character*32 directory, filename /'terrain_directory.dbs'/
c
c  Open file
c
      call getlun(infl)
      open(infl, file=filename, status='old', form='formatted',
     &   access='sequential', iostat=ierr)
c
c  Check for error opening file
c
      if (ierr .ne. 0) then
         call perror(filename)
         stop
      end if
c
c  Set next entry to read
c
      n_dir = 1
   10 continue
c
c  Read next entry
c
      read(infl, 800, end=400, err=300) directory, longitude_start
c
c  Check for too many entries
c
      if (n_dir .gt. MAX_DIR) then
         write(0, 910) filename(1:lnblnk(filename)), MAX_DIR
         stop
      end if
c
c  Store directory name and recyle
c
      dir_list(n_dir) = directory
      lon_dir(n_dir) = longitude_start
      n_dir = n_dir + 1
      goto 10
c
c  Error reading file
c
  300 continue
      write(0, 920)
      stop
c
c  Adjust counter, close file and return
c
  400 continue
      n_dir = n_dir - 1
      close(infl)
c
  500 continue
      return
c
  800 format(a32, 1x, i3)
  910 format('Too many entries in ', a, '.  Limit is', i3,
     &   ' directories')
  920 format('Error reading file terrain_directory.dbs')
      end
c  %W% %G%
c
c***********************************************************************
c
c                           get_terrain_index
c
c  Given the geographic coordinates of a point in decimal degrees, this
c  routine returns the index to the array where the parameters of the
c  7.5 minute block are stored and the number of the 7.5 minute data
c  file within the 1 degree block of data (files named as follows:
c  ter_latxlon_nr.dbs, where lat and lon are integer lat and lon,
c  and nr ranges from 1 to 64).
c
c  Example: the file for 40.21 and 74.23 would be ter_40x074_10.dbs,
c  where nr=10 is 8*int(0.23/0.125) + int(0.21/0.125) + 1.
c
c***********************************************************************
c
      subroutine get_terrain_index(d_lat, d_lon, num_f, index)
      include 'terrain_indices.inc'	!defines the integer FIRST_BLOCK
c
      i1 = int(d_lon) * 5760
      i2 = int(d_lat) * 64
      i3 = 8 * (int((d_lon - int(d_lon)) / 0.125))
      i4 = int((d_lat - int(d_lat)) / 0.125)
c
c  FIRST_BLOCK is the offset to the first lat/lon box.  When int(d_lat)
c  and int(d_lon) are equal respectively to the starting latitude and
c  longitude, the index will be 1.
c
      num_f = i3 + i4 + 1
      index = i1 + i2 + num_f - FIRST_BLOCK
      index = min(index, LAST_IDX+1)
      return
      end
c
      integer function lat_from_index(index)
      include 'terrain_indices.inc'
      iblock = FIRST_BLOCK + index - 1
      lat_from_index = mod(int(iblock/64), 90)
      return
      end
c
      integer function lon_from_index(index)
      include 'terrain_indices.inc'
      lat = lat_from_index(index)
      iblock = FIRST_BLOCK + index - 1
      lon_from_index = (iblock - 64*lat)/64/90
      return
      end
c  %W% %G%
c
c***********************************************************************
c
c                           interpolat_4_elevations
c
c  This routine finds the elevation at point pt_lat, pt_lon by
c  interpolation of the elevations at four corners of a box surrounding
c  the point.
c
c***********************************************************************
c
      subroutine interpolate_4_elevations(pt_lat, pt_lon,
     &   pt_corner_lat, pt_corner_lon, corner_el, elev )
      double precision pt_lat, pt_lon
      double precision pt_corner_lat(4), pt_corner_lon(4)
c
      double precision pts_per_deg_lat, pts_per_deg_lon
      real corner_el(4)
c
c  Check for bad elevations
c
      elev = -10000.0
      do i=1, 4
         if (corner_el(i) .lt. -999.0) goto 500
      end do
c
c  Set points per degree
c
      pts_per_deg_lat = 1200.0
      pts_per_deg_lon = 1200.0
c
c  Find difference in "distance" from corners
c
      y1 = (pt_lat - pt_corner_lat(1)) * pts_per_deg_lat
      x1 = (pt_lon - pt_corner_lon(1)) * pts_per_deg_lon
      y2 = 1.0 - y1
      x2 = 1.0 - x1
c
c  Interpolate
c
      elev = corner_el(1) * x2 * y2
      elev = elev + corner_el(4) * x1 * y2
      elev = elev + corner_el(3) * x1 * y1
      elev = elev + corner_el(2) * x2 * y1
c
  500 return
      end
c  %W% %G%
c
c***********************************************************************
c
c                           load_75_terrain
c
c  This routine reads a 7.5 minute block of terrain data into the array
c  i_elev.  The inputs to the routine are the coordinates of the
c  southeast corner of the desired block.  The first element of the
c  output array, i_elev(m, 1), is set to -10000 if no terrain data are
c  available for the m-th block.
c 
c  MAX_TER_BLK is the maximum number of 7.5' terrain data blocks that
c  can be in memory at one time.
c
c***********************************************************************
c
      subroutine load_75_terrain(se_lat, se_lon)
      include 'global.inc'
c
c  Local variables
c
      character saved_routine_name*32
      integer*2 ns_bias, ew_bias, num_bits, elevation_bias
      integer*2 c(22500), d(16875), e(11250)
      integer*2 i_t
c
c  Equate routine_name for error reporting
c
      saved_routine_name = routine_name
      routine_name = 'load_75_terrain'
c
c  Update the number of blocks in memory.  If no memory block is
c  available, replace the oldest.
c
      if (i_time(MAX_TER_BLK) .eq. 0) then
         i_next = 1
         do while (i_time(i_next) .ne. 0)
            i_next = i_next + 1
         end do
      else
         i_old = next_time	!Start with largest possible value
         do i=1, MAX_TER_BLK
            if (i_time(i) .lt. i_old) then
               i_old = i_time(i)
               i_next = i
            end if
         end do
      end if
c
c  Save memory location of the terrain block currently of interest
c
      next_pt = i_next
      m = next_pt
c
c  Initialize elevations to -10000
c
      i_elev(m, 1) = -10000.0
c
c  Get index and file offset
c
      call get_terrain_index(se_lat, se_lon, num_f, idx)
c
c  Update the list of 7.5' blocks retained in memory.
c
      if (terrain_index(m) .ne. 0) terrain_mem(terrain_index(m)) = 0
      terrain_index(m) = idx
      terrain_mem(idx) = m
c
c  Get parameters for file
c
      call get_75_parms(idx, ns_bias, ew_bias, elevation_bias,
     &   num_bits)
c
c  If num_bits = 0 then no terrain data is available for this 7.5'
c  block.  However the value of -10000 has been stored in i_elev(m, 1)
c  to flag this, so do not adjust the counter.
c
      if (num_bits .eq. 0) goto 500
c
c  Numbits = 1 is a flag indicating that all points in the 7.5 minute
c  block are 0 and therefore a data file was not created.  The elevations
c  for this block must be filled in.
c
      if (num_bits .eq. 1) then
         do i=1, 22500
            i_elev(m, i) = 0.0
         end do
         goto 500
      end if
c
c  Set record length for retrieval of data not presently in memory
c
      n_recl = (float(num_bits / 8.0)) * 22500
c
c  Open file with data for this 7.5 minute data block
c
      call open_ter_file(se_lat, se_lon, num_f, n_recl, i_ter_fl)
      if (i_ter_fl .eq. -1) then
         i_elev(m, 1) = -10000.0
         goto 500
      end if
c
c  Deal with whether elevations were stored in 8, 12, or 16 bit words.
c
      if (num_bits .eq. 16) read(i_ter_fl, rec=1) c
      if (num_bits .eq. 12) read (i_ter_fl, rec=1) d
      if (num_bits .eq. 8) read (i_ter_fl, rec=1) e
      close(i_ter_fl)
      n_points = 22500
c
c  if 16 bits
c
      if (num_bits .eq. 16) then
         do i=1, n_points
            i_elev(m, i) = c(i) + elevation_bias
         end do
         goto 500
      end if
c
c  if 12 bits
c
      if (num_bits .eq. 12) then
         n_points = n_points * 0.75
         n = 0
         do i=1, n_points, 3
            n = n + 1
            i_elev(m, n) = ishft(d(i), 4)
            i_elev(m, n) = ishft(i_elev(m, n), -4) + elevation_bias
            n = n + 1
            i_elev(m, n) = ishft(d(i), -12)
            i_t = ishft( d(i+1), 8)
            i_t = ishft ( i_t, -4)
            i_elev(m, n) = ior(i_t, i_elev(m, n)) + elevation_bias
            n = n + 1
            i_elev(m, n) = ishft(d(i+1), -8)
            i_t = ishft(d(i+2), +12)
            i_t = ishft(i_t, -4)
            i_elev(m, n) = ior(i_t, i_elev(m, n)) + elevation_bias
            n = n + 1
            if (n*m .gt. MAX_TER_BLK*22500) then
               write (0, '(a,i4,a,i6,a)') 'Cannot store', m,
     &            ' blocks of', n, ' points in array i_elev'
               stop
            end if
            i_elev(m, n) = ishft(d(i+2), -4) + elevation_bias
         end do
         goto 500
      end if
c
c  if 8 bits
c
      if (num_bits .eq. 8) then
         n_points = n_points * 0.5
         n = 0
         do i=1, n_points
            n = n + 1
            i_elev(m, n) = ishft(e(i), 8)
            i_elev(m, n) = ishft(i_elev(m, n), -8) + elevation_bias
            n = n + 1
            i_elev(m, n) = ishft(e(i), -8) + elevation_bias
         end do
         goto 500
      end if
c
  500 continue
      routine_name = saved_routine_name
      return
      end

      subroutine open_ter_file(se_lat, se_lon, num_f, n_recl, i_ter_fl)
      include 'global.inc'
      character file_name*52
c
      istat = -1
      i_dir = 1
      do while (i_dir .le. MAX_DIR .and. istat .ne. 0)
c
c  Get file name for this 7.5 minute data block
c
         if (int(se_lon) .ge. lon_dir(i_dir)) then
            l = lnblnk(dir_list(i_dir))
            write(file_name, 900) dir_list(i_dir)(1:l), int(se_lat),
     &         int(se_lon), num_f
c
c  Open next file in target range of longitudes
c
            call getlun(i_ter_fl)
            open(unit=i_ter_fl, file=file_name, status='old',
     &         iostat=istat, access='direct', form='unformatted',
     &         recl=n_recl)
         end if
         i_dir = i_dir + 1
      end do
c
c  Set logical unit number to -1 in case file has not been found
c
      if (istat .ne. 0) then
         write(0, '(a,1x,a,1x,a)') file_name(1:lnblnk(file_name)),
     &      'not found by', routine_name(1:lnblnk(routine_name))
         i_ter_fl = -1
      end if
      return
c
  900 format(a, 'ter_', i2.2, 'x', i3.3, '_', i2.2, '.dbs')
      end
c
c************************************************************************
c
c                           rd_main
c
c  This routine reads one record from the main TV data base
c
c************************************************************************
c
      subroutine rd_main(imain, irec)
      include 'global.inc'
c
      character svc_cls*2, cty*1, state*2, city*20, call*8
      character pol*1, da*1, beamt*1, zone*1, antmnf*3
      character antmod*14, off*1, arn*8
      common /mainrec/
     &   latsec, lonsec, erp, haat, rcamsl, ref_az, itype, istat,
     &   svc_cls, off, pol, da, beamt, zone, antmnf, antmod,
     &   ichan, cty, city, state, call, arn
c
c  Read record
c
      read(imain, rec=irec, end=450, err=400)
     &   ichan, cty, city, state, call, latsec, lonsec,
     &   erp, haat, rcamsl, ref_az, itype, istat, svc_cls,
     &   off, pol, da, beamt, zone, antmnf, antmod, arn
c
c  Adjust call sign for 3 letter US
c
      if (cty .eq. 'A') then
        if (call(4:5) .eq. 'TV') call(4:8) = '     '
        if (call(4:5) .eq. '-T') call(4:8) = '     '
      end if
      goto 500
c
c  Error reading data base
c
  400 continue
      write(0, '(a,i5)') 'Error reading main data base at record',
     &   irec
      call exit(257)
c
c  End of file while reading data base
c
  450 continue
      city = 'end of file'
  500 return
      end
c  %W% %G%
c
c***********************************************************************
c
c                        load_ter_index
c
c  This routine loads the index to the 7.5 minute terrain blocks.
c
c  The 4 variables for each block are stored sequentially in the
c  array index_75_ter.  These variables are north/south bias,
c  east/west bias, elevation bias, and number of bits used to store
c  data.  A dummy block with these variables set to 0 is added at
c  the end.
c
c***********************************************************************
c
      subroutine load_ter_index
      include 'global.inc'
c
      parameter(NUM_BYTES = 2*NUM_TERRAIN_INDEX_ELEM)
      character index_file*64
      integer*2 idummy(NUM_TERRAIN_INDEX_ELEM)
      equivalence(idummy, index_75_ter)
c
c  Determine location of index file, and set path and name
c
      index_file = dir_list(HOME_PARTITION)
      l = lnblnk(index_file)
      index_file = index_file(1:l) // 'terrain_index.dbs'
c
c  Recl is the size in bytes of the entire index file
c
      call getlun(idx)
      open(unit=idx, file=index_file, access='direct',
     &   form='unformatted', recl=NUM_BYTES, iostat=ierr)
      if (ierr .ne. 0) then
         call perror(index_file)
         stop 'Unable to open terrain index file'
      end if
      read(idx, rec=1, err=900) idummy
c
c  Set the last four 2-byte integers to 0 to indicate that the last 7.5
c  minute terrain block contains no data.
c
      i = NUM_TERRAIN_INDEX_ELEM + 1
      do while (i .le. NUM_TERRAIN_INDEX_ELEM + 4)
         index_75_ter(i) = 0
         i = i + 1
      end do
      return
c
  900 continue
      call perror(index_file)
      stop 'Unable to read index of terrain data files'
      end
c  %W% %G%
c
c**************************************************************
c
c                      sec_to_deg
c
c  Convert seconds to decimal degrees, or seconds to degrees
c  in double precision.
c
c**************************************************************
c
      subroutine sec_to_deg(isec, deg)
      ix = abs(isec)
      deg = float(ix) / 3600.0
      if (isec .lt. 0) deg = -deg
      return
      end

      subroutine sec_to_dpdeg(isec, dpdeg)
      double precision dpdeg
      ix = abs(isec)
      dpdeg = dble(ix) / dble(3600)
      if (isec .lt. 0) dpdeg = -dpdeg
      return
      end
c  %W% %G%
c
c***********************************************************************
c
c                            init_ter_mem
c
c  This routine initializes the array terrain_mem to indicate that no
c  terrain data is yet in memory.  The terrain_mem array contains an
c  index to the array i_elev where the terrain elevations for a specific
c  7.5' block of terrain data are stored.  Terrain_men(idx) = 0 means
c  that the 7.5' block with index idx is not in memory.  Terrain block
c  indices run from 0 to LAST_IDX+1, and there is an index for blocks
c  all the way up to the north pole at all longitudes except possibly
c  the last and most western.  The terrain block with index LAST_IDX+1
c  is a dummy with parameters set to indicate no data.
c
c***********************************************************************
c
      subroutine init_ter_mem
c
      include 'global.inc'
c
c  Initialize array to 0.  LAST_IDX is last possible index for the 7.5'
c  terrain blocks found in the terrain data directories.  The latter are
c  identified by subroutine get_ter_dirs.
c
      do i=1, LAST_IDX+1
         terrain_mem(i) = 0
      end do
c
c  Set initial time stamp to 0
c
      next_time = 0
c
c  Set time stamps to 0
c
      do i=1, MAX_TER_BLK
         i_time(i) = 0
      end do
c
      return
      end
c  %W% %G%
c
c***********************************************************************
c
c                          get_75_parms
c
c  This routine gets the north/south bias, east/west bias, elevation
c  bias, and number of bits used to store elevations for a 7.5 minute
c  terrain block at the index given by the argument idx.  The latter
c  ranges from 1 to LAST_IDX+1, and the last block is a dummy terminator
c  with its parameters set to 0.
c
c***********************************************************************
c
      subroutine get_75_parms(idx, ns_bias, ew_bias, elevation_bias,
     &   num_bits)
      integer*2 ns_bias, ew_bias, num_bits, elevation_bias
      include 'global.inc'
c
      if (idx .gt. LAST_IDX+1) stop
     &   'Index of terrain blocks out of bounds'
      j = 1 + ((idx - 1) * 4)
      ns_bias = index_75_ter(j)
      ew_bias = index_75_ter(j+1)
      elevation_bias = index_75_ter(j+2)
      num_bits = index_75_ter(j+3)
      return
      end
c
c***********************************************************************
c
c                            get_rad_haat
c
c  This routine computes the haat for a given location and azimuth
c  based on the method specified in the FCC R&R.  If terrain data
c  is not found at a point along the desired path that point is
c  ignored.  If more than half the points have errors the routine
c  will return a value of -10000.0.  Also if switch compute_haat is
c  set to false the routine returns a value of -10000. Terrain
c  elevations are retrieved every 0.1 km between 3.2 and 16.1 km.
c
c  The inputs to the routine are the azimuth in degrees (az), the
c  RCAMSL in meters and the site coordinates in double precision
c  degrees (dlat1, dlon1).  The output is in meters.
c
c***********************************************************************
c
      subroutine get_rad_haat(az, amslrc, dlat1, dlon1, avg_elev,
     &   rad_haat)
      double precision dlat1, dlon1, dlat2, dlon2
      include 'global.inc'
c
c  Local variables
c
      character saved_routine_name*32
      real pt_elevation(150)
c
c  Equate routine_name for error reporting
c
      saved_routine_name = routine_name
      routine_name = 'get_rad_haat'
c
c  If HAATs are not to be computed, return with rad_haat and avg_elev set
c  to -10000
c
      avg_elev = -10000.0
      rad_haat = -10000.0
      if (.not. compute_haat) goto 500
c
c  Get coordinates of points along path.  Terrain data is to be retrieved
c  at 0.1 km intervals between 3.2 and 16.1 km from the site along the
c  specified azimuth.
c
      n_ter_pt = 0
      d_inc = 0.1
      adist = 3.1
c
c  Loop over distance increments
c
   10 continue
      adist = adist + d_inc
      if (adist .gt. 16.11) goto 400
      call calcrd_r2(dlat1, dlon1, adist, az, dlat2, dlon2)
      rlat = dlat2
      rlon = dlon2
c
c  Get NAD83 coordinates of point
c
      call get_nad_83_coord(rlat, rlon, rlat_83, rlon_83, n_error_code)
c
c  Report error and flag point
c
      if (n_error_code .ne. 0) then
         write(0, 900) rlat, rlon, routine_name(1:lnblnk(routine_name))
         elev = -10000.0
         goto 50
      end if
c
c  Get terrain elevation
c
      call get_point_elevation(rlat_83, rlon_83, elev)
c
   50 continue
      n_ter_pt = n_ter_pt + 1
      pt_elevation(n_ter_pt) = elev
      goto 10
c
c  Compute average elevation and subtract from RCAMSL.  If value is
c  less than 30.5 m, set HAAT to 30.5.
c
  400 continue
      n = 0
      sum = 0.0
      do 100 i=1, n_ter_pt
         if (pt_elevation(i) .eq. -10000.0) goto 100
         n = n + 1
         sum = sum + pt_elevation(i)
  100 continue
c
      if (n .lt. n_ter_pt/2) goto 500
      avg_elev = sum/n
      rad_haat = amslrc - avg_elev
      if (rad_haat .lt. 30.5) rad_haat = 30.5
c
  500 continue
      routine_name = saved_routine_name
      return
c
  900 format('Error retrieving terrain data at ', f9.4, 1x, f9.4,
     &   ' in routine ', a)
      end
c  %W% %G%
c
c**************************************************************************
c
c				CALCRD_R2
c
c  This routine calculates the coordiantes of a point using the RIO-81
c  agreement method. The inputs to the routine are the latitude and
c  longitude of the point in decimal degrees, the distance and the
c  azimuth passed as arguments. The distance is in kilometers and the
c  azimuth is in degrees. The routine returns coordinate in degrees.
c
c		  ************ IMPORTANT ***********
c
c  The latitude and longitude are DOUBLE precision
c
c**************************************************************************
c
      subroutine CALCRD_R2(D_LAT1, D_LON1, DISTANCE,
     &   AZ, RLAT2, RLON2)
      DOUBLE PRECISION D_LAT1, D_LON1, F1, F2, F3, F4, F5, F6, f7
      DOUBLE PRECISION RLAT1, RLON1, RLAT2, RLON2, DRADIAN, DD, AZD
      include 'global.inc'
c
      character saved_routine_name*32
c
      saved_routine_name = routine_name
      routine_name = 'calcrd_r2'
c
c  Check for distance approaching 0.  Errors occur if distance is too
c  small.
c
      if (distance .lt. 0.001) then
         rlat2 = d_lat1
         rlon2 = d_lon1
         goto 110
      end if
      C1 = degkm
c
c  Convert coordinates to radians
c
      DRADIAN = RADIAN
      DD = (DISTANCE / C1) * DRADIAN
      AZD = AZ * DRADIAN
      RLAT1 = D_LAT1 * RADIAN
      RLON1 = D_LON1 * RADIAN
      F1 = DSIN(RLAT1) * DCOS(DD)
      F2 = DCOS(RLAT1) * DSIN(DD) * DCOS(AZD)
      F3 = DASIN (F1 + F2)
      IF (AZ .EQ. 0.0 .OR. AZ .EQ. 180.0) THEN
      RLON2 = RLON1 * DEGREE
      GO TO 100
      END IF
      F4 = (DCOS(DD) - (DSIN(RLAT1) * DSIN(F3)))
      F5 = DCOS(RLAT1) * DCOS(F3)
      f7 = f4 / f5
c
c  Trap out of range values
c
      if (f7 .lt. -1.0) f7 = -1.0
      if (f7 .gt. 1.0) f7 = 1.0
      F6 = DACOS(f7)
      IF (AZ .LT. 180.0) F6 = -F6
      RLON2 = (RLON1 + F6) * DEGREE
  100 continue
      RLAT2 = F3 * DEGREE
  110 continue
      routine_name = saved_routine_name
      RETURN
      END
c  %W% %G%
c
c***********************************************************************
c
c                        get_point_elevation
c
c  Given a point in NAD83 coordinates get the elevation by interpolation
c  of the elevations of the elevations that are at the corner of a
c  3" box containing the point.  Routine get_3sec_elevation does the
c  actual retrieval of the elevations at the 3" corner points.
c
c***********************************************************************
c
      subroutine get_point_elevation(pt_lat, pt_lon, elev)
      include 'global.inc'
c
      double precision d_pt_lat, d_pt_lon, d_cor_lat(4), d_cor_lon(4)
      logical in_water
      real c_elev(4)
c
c  Get corners of 3" box containing point
c
      d_pt_lat = pt_lat
      d_pt_lon = pt_lon
      call get_3sec_corners(d_pt_lat, d_pt_lon, d_cor_lat,
     &   d_cor_lon)
c
c  Get elevations of corner points
c
      do i=1, 4
         call get_3sec_elevation(d_cor_lat(i), d_cor_lon(i),
     &      c_elev(i))
c
c  Return a large negative value for the elevation if there is no
c  terrain data for one of these corners, unless the corner point is
c  in the water.  Set the elevation of water points to 0.0 so that
c  terrain profiles will not be interrupted.
c
         if (c_elev(i) .eq. -10000.0) then
            x_lat = d_cor_lat(i)
            x_lon = d_cor_lon(i)
            call ck_in_water(x_lat, x_lon, in_water)
            if (in_water) then
               c_elev(i) = 0.0
            else 
               elev = -10000.0
               goto 500
            end if
         end if
      end do
c
c  Interpolate to get elevation at actual point
c
      call interpolate_4_elevations(d_pt_lat, d_pt_lon,
     &   d_cor_lat, d_cor_lon, c_elev, elev )
c
  500 return
      end
c  %W% %G%
c
c***********************************************************************
c
c                       ck_in_water
c
c  Determine whether a given point lies in water.  These tests rely upon
c  the preclusion of points for which elevation data is contained in the
c  U.S. terrain data base.  The points in a test block for which in_water
c  is true are either points which truely lie in water or else are points
c  which will never be presented to this routine because terrain data
c  will have been found for them.
c
c***********************************************************************
c
      subroutine ck_in_water(xlat, xlon, in_water)
      logical in_water, error
c
c  Pacific Coast
c
      in_water = xlat .gt. 32.5 .and. xlat. le. 49.0
     &   .and. xlon .ge. 117.125
      if (in_water) return
c
c  Atlantic Coast and Gulf of Mexico
c
      in_water = xlat .ge. 31.0 .and. xlat .lt. 49.0
     &   .and. xlon .lt. 82.0
      in_water = in_water .or. (xlat .lt. 31.0 .and. xlon .lt. 97.375)
      if (in_water) return
c
c  Points which fail the preceding tests are either in Mexico, or in
c  Mexican coastal waters, or north of the U.S. beyond availability of
c  Canadian terrain data.  This conclusion presumes that the routine
c  has been called after first determining that the point in question
c  is not covered by the U.S. terrain data base.
c
c  Test for improper call of this subroutine or disk IO failure
c
      error = xlat .gt. 33.0 .and. xlat .le. 49.0
     &   .and. xlon .lt. 117.125 .and. xlon .ge. 98.0
      error = error .or. (xlat .ge. 31.0 .and. xlat .lt. 49.0
     &   .and. xlon .lt. 97.375)
      if (error) then
         write(0, '(a,f7.3,a1,f8.3,a)') 'ck_in_water: point',
     &      xlat, ',', xlon,
     &      ' should have been found in terrain data base'
c        call exit(257)
      end if
      return
      end
      integer function handler(sig, sip, uap)
c
c  %W% %G%
c
      integer sig, address, code
c
      structure /fault/
           integer address
      end structure
c
      structure /siginfo/
           integer si_signo
           integer si_code
           integer si_errno
           record /fault/ fault
      end structure
      record /siginfo/ sip
c
      character routine_name*32
      common /current_routine/ routine_name
c
c  The following definitions of exception codes 1 through 8 are found
c  in /usr/include/sys/machsig.h:
c
      character*32 meaning(8) /
     &   'integer divide by zero',
     &   'integer overflow',
     &   'floating point divide by zero',
     &   'floating point overflow',
     &   'floating point underflow',
     &   'floating point inexact result',
     &   'invalid floating point operation',
     &   'subscript out of range'/
c
      call flush(6)
      address = sip.fault.address
      code = sip.si_code
      write(0, 1000) address, routine_name(1:lnblnk(routine_name))
      if (code .le. 8) then
         write(0, 1002) code, meaning(code)
      else
         write(0, '(a,i6)') 'Unrecognized exception code', code
      end if
      call flush(0)
      return
c
 1000 format('IEEE exception occurred at address X', Z8.8,
     &   ', routine ', a)
 1002 format('Exception code', i3, ', ', a)
      end
